@@ -56,7 +56,7 @@ pub struct MqttClient<
     const MSG_SIZE: usize,
     const MSG_COUNT: usize,
 > {
-    pub(crate) network: InterfaceHolder<TcpStack>,
+    pub(crate) network: InterfaceHolder<TcpStack, MSG_SIZE>,
     clock: Clock,
     session_state: SessionState<Clock, MSG_SIZE, MSG_COUNT>,
     connection_state: StateMachine<Context>,
@@ -126,6 +126,9 @@ where
             _ => {}
         }
 
+        // Attempt to finish any pending packets.
+        self.network.finish_write()?;
+
         self.handle_timers()?;
 
         Ok(())
@@ -175,7 +178,9 @@ where
         &mut self,
         interval_seconds: u16,
     ) -> Result<(), Error<TcpStack::Error>> {
-        if self.connection_state.state() != &States::Active {
+        if (self.connection_state.state() == &States::Active)
+            || (self.connection_state.state() == &States::Establishing)
+        {
             return Err(Error::NotReady);
         }
 
@@ -201,6 +206,11 @@ where
             return Err(Error::NotReady);
         }
 
+        // We can't subscribe if there's a pending write in the network.
+        if self.network.has_pending_write() {
+            return Err(Error::NotReady);
+        }
+
         let packet_id = self.session_state.get_packet_identifier();
 
         let mut buffer: [u8; MSG_SIZE] = [0; MSG_SIZE];
@@ -246,6 +256,12 @@ where
     /// # Returns
     /// True if the client is able to service QoS 1 requests.
     pub fn can_publish(&self, qos: QoS) -> bool {
+        // We cannot publish if there's a pending write in the network stack. That message must be
+        // completed first.
+        if self.network.has_pending_write() {
+            return false;
+        }
+
         self.session_state.can_publish(qos)
     }
 
@@ -271,14 +287,15 @@ where
         retain: Retain,
         properties: &[Property],
     ) -> Result<(), Error<TcpStack::Error>> {
-        // TODO: QoS 2 support.
-        assert!(qos != QoS::ExactlyOnce);
-
         // If we are not yet connected to the broker, we can't transmit a message.
         if self.is_connected() == false {
             return Ok(());
         }
 
+        if !self.can_publish(qos) {
+            return Err(Error::NotReady);
+        }
+
         debug!(
             "Publishing to `{}`: {:?} Props: {:?}",
             topic, data, properties
@@ -351,6 +368,11 @@ where
 
         // Replay QoS 1 messages
         for key in self.session_state.pending_publish_ordering.iter() {
+            // If the network stack cannot send another message, do not attempt to send one.
+            if self.network.has_pending_write() {
+                break;
+            }
+
             let message = self.session_state.pending_publish.get(&key).unwrap();
             self.network.write(message)?;
         }
@@ -437,6 +459,11 @@ where
             return Ok(());
         }
 
+        // If there's a pending write, we can't send a ping no matter if it is due.
+        if self.network.has_pending_write() {
+            return Ok(());
+        }
+
         let now = self.clock.try_now()?;
 
         // Note: The ping timeout is set at this point so that it's running even if we fail
